<?php

namespace Drupal\mailjet_webform_subscription;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Mailjet\Client;
use Mailjet\Resources;

/**
 * Service to connect to mailjet api.
 */
class MailjetApiWrapper implements IMailjetApiWrapperInterface {

  use StringTranslationTrait;

  /**
   * Configuration object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $mailjetApiConfig;
  /**
   * Mailjet client.
   *
   * @var \Mailjet\Client
   */
  protected $mailJetApiClient;

  /**
   * The mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * Drupal logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $user;

  public function __construct($mailjetApiConfig, $mailManager, $logger, $user) {
    $this->mailjetApiConfig = $mailjetApiConfig;
    $this->mailManager = $mailManager;
    $this->logger = $logger;
    $this->user = $user;

    $this->mailJetApiClient = new Client(
      $this->mailjetApiConfig->get('api_key_public'),
      $this->mailjetApiConfig->get('api_key_secret'),
      TRUE,
      ['version' => 'v3']
    );
  }

  /**
   * {@inheritdoc}
   */
  public function appendToList(string $emailAddress, string|null $firstName, string|null $lastName, int $listId) : bool {
    $fullName = implode(' ', array_filter([$firstName, $lastName]));

    $body = [
      'Action' => 'addnoforce',
      'Email' => $emailAddress,
    ];

    if ($firstName) {
      $body['Properties']['firstname'] = $firstName;
    }
    if ($lastName) {
      $body['Properties']['lastname'] = $lastName;
    }
    if ($fullName) {
      $body['Name'] = $fullName;
    }

    $response = $this->mailJetApiClient->post(Resources::$ContactslistManagecontact, ['id' => $listId, 'body' => $body]);

    if (!$response->success()) {
      $this->logger->error($this->t(
        'Error appending @user_email to list @list_id. @message',
        ['@user_email' => $emailAddress, '@list_id' => $listId, '@message' => $response]
      ));
    }

    return $response->success();
  }

  /**
   * {@inheritdoc}
   */
  public function sendTemplateEmail(string $recipientEmail, int $templateId, array $variables = []): void {
    $templateDetails = $this->getMailJetTemplateDetails($templateId);

    if (!$templateDetails) {
      return;
    }

    if (!$templateDetails['Subject'] || !$templateDetails['From']) {
      $this->logger->error(
        $this->t(
          'Mailjet Template missing "Subject" and/or "From" param/s. Template ID: @templateId, Email:@email',
          [
            '@templateId' => $templateId,
            '@email' => $recipientEmail,
          ]
        )
      );
      return;
    }

    $this->mailManager->mail(
      'mailjet_webform_subscription',
      'send_using_mailjet_template',
      $recipientEmail,
      $this->user->getPreferredLangcode(),
      [
        'subject' => $templateDetails['Subject'],
        'from' => $templateDetails['SenderEmail'],
        'params' => [
          'TemplateId' => $templateId,
          'Variables' => $variables,
        ],
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getMailJetLists() : array {
    $response = $this->mailJetApiClient->get(Resources::$Contactslist, ['filters' => ['limit' => 1000]]);

    if (!$response->success()) {
      throw new \Exception('Failed to fetch mailjet lists');
    }

    $result = [];
    foreach ($response->getData() as $list) {
      $result[$list['ID']] = $list['Name'] . ' (' . $list['ID'] . ')';
    }

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function getMailjetTemplates() : array {
    $response = $this->mailJetApiClient->get(Resources::$Template, ['filters' => ['limit' => 1000]]);

    if (!$response->success()) {
      throw new \Exception('Failed to fetch mailjet templates');
    }

    $result = [];
    foreach ($response->getData() as $template) {
      $result[$template['ID']] = $template['Name'] . ' (' . $template['ID'] . ')';
    }

    return $result;
  }

  /**
   * Get extra information about the template.
   */
  private function getMailJetTemplateDetails(string $templateId) : array | bool {
    $response = $this->mailJetApiClient->get(Resources::$TemplateDetailcontent, ['id' => $templateId]);

    if (!$response->success()) {
      $this->logger->error(
        $this->t(
          'Failed to fetch template details. Template ID: @templateId',
          [
            '@templateId' => $templateId,
          ]
        )
      );
      return FALSE;
    }

    $data = $response->getData();

    if (count($data) < 1) {
      $this->logger->error(
        $this->t(
          'Specified template not found. Template ID: @templateId',
          [
            '@templateId' => $templateId,
          ]
        )
      );
      return FALSE;
    }

    return $data[0]['Headers'];
  }

}
